# Copyright 2022 The KerasCV Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import tensorflow as tf

import keras_cv


class VisualizeObjectDetectionPredictions(tf.keras.callbacks.Callback):
    def __init__(self, x, y, value_range, bounding_box_format, artifacts_dir=None):
        self.x = x
        self.y = y
        self.artifacts_dir = artifacts_dir
        self.value_range = value_range
        self.bounding_box_format = bounding_box_format
        keras_cv.utils.ensure_exists(artifacts_dir)

    def on_epoch_end(self, epoch, logs):
        y_pred = self.model.predict(self.x)
        path = (
            None if self.artifacts_dir is None else f"{self.artifacts_dir}/{epoch}.png"
        )
        keras_cv.visualization.plot_bounding_box_gallery(
            self.x,
            value_range=self.value_range,
            bounding_box_format=self.bounding_box_format,
            y_true=self.y,
            y_pred=y_pred,
            path=path,
        )



import tensorflow as tf
import numpy as np


def to_numpy(x):
    if x is None:
        return None

    if isinstance(x, tf.RaggedTensor):
        x = x.to_tensor(-1)

    if isinstance(x, tf.Tensor):
        return x.numpy()
    if isinstance(x, (np.ndarray, np.generic)):
        return x
    return np.array(x)



import os
from datetime import datetime

from luketils.utils.to_numpy import to_numpy
from luketils.utils.experiment_name import experiment_name
from luketils.utils.ensure_exists import ensure_exists



import keras_cv
import matplotlib.pyplot as plt
import tensorflow as tf


def plot_gallery(
    images, value_range, rows=3, columns=3, scale=2, path=None, show=None, cols=None
):
    """gallery_show shows a gallery of images.

    Args:
        images: a Tensor or NumPy array containing images to show in the gallery.
        value_range: value range of the images.
        rows: number of rows in the gallery to show.
        columns: number of columns in the gallery to show.
        scale: how large to scale the images in the gallery
        path: (Optional) path to save the resulting gallery to.
        show: (Optional) whether or not to show the gallery of images.
        cols: (Optional) alias for columns
    """
    if path is None and show is None:
        # Default to showing the image
        show = True
    if path is not None and show:
        raise ValueError(
            "plot_gallery() expects either `path` to be set, or `show` " "to be true."
        )

    fig = plt.figure(figsize=(columns * scale, rows * scale))
    fig.tight_layout()  # Or equivalently,  "plt.tight_layout()"
    plt.subplots_adjust(wspace=0, hspace=0)
    plt.margins(x=0, y=0)
    plt.axis("off")

    columns = cols if cols is not None else columns

    images = keras_cv.utils.transform_value_range(
        images, original_range=value_range, target_range=(0, 255)
    )
    if isinstance(images, tf.Tensor):
        images = images.numpy()

    images = images.astype(int)
    for row in range(rows):
        for col in range(columns):
            index = row * columns + col
            plt.subplot(rows, columns, index + 1)
            plt.imshow(images[index].astype("uint8"))
            plt.axis("off")
            plt.margins(x=0, y=0)

    if path is None and not show:
        return
    if path is not None:
        plt.savefig(fname=path, pad_inches=0, bbox_inches="tight")
        plt.close()
    elif show:
        plt.show()
        plt.close()



import contextlib
from luketils.artifacts.base import get_base
from luketils.artifacts.collection import add_artifact


@contextlib.contextmanager
def file(path):
    with open(path, "w") as f:
        add_artifact(f)
        yield f



import keras_cv
from luketils import utils
from luketils import drawing
from luketils.visualization.plot_gallery import plot_gallery
import functools


def plot_bounding_box_gallery(
    images,
    value_range,
    bounding_box_format,
    pred_color=(255, 255, 0),
    true_color=(0, 255, 255),
    y_true=None,
    y_pred=None,
    thickness=2,
    font_scale=1.0,
    text_thickness=None,
    class_mapping=None,
    **kwargs
):
    """plots a gallery of images with corresponding bounding box annotations

    Args:
        images: a Tensor or NumPy array containing images to show in the gallery.
        value_range: value range of the images.
        bounding_box_format: the bounding_box_format  the provided bounding boxes are
            in.
        y_true: a Tensor or RaggedTensor representing the ground truth bounding boxes.
        y_pred: a Tensor or RaggedTensor representing the predicted truth bounding
            boxes.
        pred_color: three element tuple representing the color to use for plotting
            predicted bounding boxes.
        true_color: three element tuple representing the color to use for plotting
            true bounding boxes.
        class_mapping: (Optional) class mapping from class IDs to strings
        thickness: (Optional) thickness for the box and text labels.  Defaults to 2.
        text_thickness: (Optional) the thickness for the text, defaults to `1.0`.
        font_scale: (Optional) font size to draw bounding boxes in.
        kwargs: keyword arguments to propagate to
            `keras_cv.visualization.gallery_show()`.
    """
    images = utils.to_numpy(images)
    y_true = utils.to_numpy(y_true)
    y_pred = utils.to_numpy(y_pred)

    plotted_images = images

    draw_fn = functools.partial(
        drawing.draw_bounding_boxes,
        bounding_box_format=bounding_box_format,
        class_mapping=class_mapping,
        thickness=thickness,
        text_thickness=text_thickness,
        font_scale=font_scale,
    )
    if y_pred is not None:
        plotted_images = draw_fn(
            plotted_images,
            y_pred,
            pred_color,
        )

    if y_true is not None:
        plotted_images = draw_fn(
            plotted_images,
            y_true,
            true_color,
        )

    plot_gallery(plotted_images, value_range, **kwargs)



# Copyright 2022 The KerasCV Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import keras_cv
import tensorflow as tf
import cv2
from luketils import utils
import numpy as np


def draw_bounding_boxes(
    images,
    bounding_boxes,
    color,
    bounding_box_format,
    thickness=2,
    text_thickness=1,
    font_scale=1.0,
    class_mapping=None,
):
    """draws bounding boxes on the target image.

    Args:
        images: a batch Tensor of images to plot bounding boxes onto.
        bounding_boxes: a Tensor of batched bounding boxes to plot onto the provided
            images
        color: the color in which to plot the bounding boxes
        bounding_box_format: The format of bounding boxes to plot onto the images. Refer
          [to the keras.io docs](https://keras.io/api/keras_cv/bounding_box/formats/)
          for more details on supported bounding box formats.
        thickness: (Optional) thickness for the box and text labels.  Defaults to 2.
        text_thickness: (Optional) the thickness for the text, defaults to `1.0`.
        font_scale: (Optional) scale of font to draw in.  Defaults to `1.0`.
        class_mapping: (Optional) dictionary from class ID to class label.
    Returns:
        images with bounding boxes plotted on top of them
    """
    bounding_boxes = keras_cv.bounding_box.convert_format(
        bounding_boxes, source=bounding_box_format, target="xyxy", images=images
    )
    text_thickness = text_thickness or thickness

    bounding_boxes = utils.to_numpy(bounding_boxes)
    images = utils.to_numpy(images)

    class_mapping = class_mapping or {}
    result = []

    for i in range(images.shape[0]):
        bounding_box_batch = bounding_boxes[i]
        image = images[i]
        for b_id in range(bounding_box_batch.shape[0]):
            x, y, x2, y2, class_id = bounding_box_batch[b_id][:5].astype(int)

            if class_id == -1:
                continue

            cv2.rectangle(image, (x, y), (x2, y2), color, thickness)
            class_id = int(class_id)

            if class_id in class_mapping:
                label = class_mapping[class_id]
                cv2.putText(
                    image,
                    label,
                    (x, y - 10),
                    cv2.FONT_HERSHEY_SIMPLEX,
                    font_scale,
                    color,
                    text_thickness,
                )
        result.append(image)
    return np.array(result).astype(int)



import os

from setuptools import find_packages
from setuptools import setup


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    # intentionally *not* adding an encoding option to open, See:
    #   https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    with open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")


setup(
    name="luketils",
    version=get_version("luketils/__init__.py"),
    description="",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Luke Wood",
    author_email="lukewoodcs@gmail.com",
    url="https://github.com/lukewood/luketils",
    license="Apache License 2.0",
    install_requires=["opencv-python", "tensorflow"],
    extras_require={
        "dev": [
            "flake8",
            "pytest",
            "pytype",
            "setuptools",
            "twine",
            "wheel",
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    packages=find_packages(),
)
